import gitlab
import pathlib
from urllib.parse import urlsplit

from gip import logger
from gip import exceptions
from gip.sources import base

LOG = logger.get_logger(__name__)


class Gitlab(base.Source):
    """ Gitlab source """

    def __init__(self, repo, version, token):
        """
        Inits Gitlab source

        :param repo: url to repository
        :param version: Tag, branch name or commit sha, defaults to master
        :param token: gitlab api token
        :raise exceptions.AuthenticationError: could not authenticate
        :raise exceptions.HttpError: could not connect
        :raise exceptions.RepoNotFound: repository not found
        """
        # Set version
        self.version = version or 'master'

        # Init Gitlab connection
        try:
            self.gl = gitlab.Gitlab(
                url=self._get_base_url(repo),
                private_token=token
            )
        except gitlab.GitlabAuthenticationError:
            raise exceptions.AuthenticationError(url=repo)
        except gitlab.GitlabHttpError as e:
            raise exceptions.HttpError(url=repo, code=e.response_code)

        # Split project_path from passed repo url
        project_path = self._get_project_path(repo)

        # Get project from Gitlab API as object
        try:
            self.project = self.gl.projects.get(project_path)
        except gitlab.exceptions.GitlabGetError:
            raise exceptions.RepoNotFound(repo)

    def get_archive(self, dest):
        """
        Downloads archive in dest_dir

        :param dest: path where to download the archive to
        :raise exceptions.DirectoryDoesNotExist: destination dir does not exist
        :raise exceptions.AuthenticationError: could not authenticate
        :raise exceptions.ArchiveNotFound: archive not found
        """
        # Raise error when destination directory does not exists
        dest = pathlib.Path(dest)
        if not dest.parent.is_dir():
            raise exceptions.DirectoryDoesNotExist(dest)

        # Get project archive
        try:
            with open(dest, "wb") as f:
                self.project.repository_archive(
                    sha=self.version, streamed=True, action=f.write)
        except FileNotFoundError:
            raise exceptions.DirectoryDoesNotExist(dest)
        except gitlab.GitlabAuthenticationError:
            raise exceptions.AuthenticationError(url=self.project.web_url)
        except gitlab.GitlabListError:
            raise exceptions.ArchiveNotFound(
                repo=self.project.web_url,
                version=self.version
            )

    def get_commit_hash(self):
        """
        Get commit hash for this source

        :return: commit hash for source
        """
        commits = self.project.commits.list(ref_name=self.version)
        return commits[0].id

    def _get_project_path(self, repo):
        """
        Get project path of repo url

        :return: project path of source
        """
        # Removes leading slash
        path = urlsplit(repo).path[1:]
        if path[-4:] == ".git":
            # Remove .git from URL
            path = path[:-4]
        return path

    def _get_base_url(self, repo):
        """
        Get base url of repository url

        :return: base url of Gitlab instance
        """
        # Split repo url in parts
        split_url = urlsplit(repo)
        return "{0}://{1}".format(split_url.scheme, split_url.netloc)



import pathlib
import github3

from gip import logger
from gip import exceptions
from gip.sources import base

LOG = logger.get_logger(__name__)


class Github(base.Source):
    """
    Github source
    """

    def __init__(self, repo, version, token):
        """
        Inits Github source

        :param repo: url to repository
        :param version: Tag, branch name or commit sha, defaults to master
        :param token: github api token
        :raise exceptions.AuthenticationError: could not authenticate
        :raise exceptions.HttpError: could not connect
        :raise exceptions.RepoNotFound: repository not found
        """
        # Set version
        self.version = version or 'master'

        # Init Github object
        try:
            self.gh = github3.login(token=token)
        except github3.exceptions.AuthenticationFailed:
            raise exceptions.AuthenticationError(url="https://github.com")

        # Get repository from Gitlab API
        try:
            self.repository = self.gh.repository(
                owner=self.get_owner(repo),
                repository=self.get_repo_name(repo)
            )
        except github3.exceptions.ConnectionError:
            raise exceptions.HttpError(url=repo)
        except github3.exceptions.NotFoundError:
            raise exceptions.RepoNotFound(repo=repo)

    def get_archive(self, dest):
        """
        Downloads archive in dest_dir

        :param dest: path where to download the archive to
        :raise exceptions.DirectoryDoesNotExist: destination dir does not exist
        :raise exceptions.ArchiveNotFound: archive not found
        """
        # Raise error when destination directory does not exists
        dest = pathlib.Path(dest)
        if not dest.parent.is_dir():
            raise exceptions.DirectoryDoesNotExist(dest)

        # Download repository archive to dest
        result = self.repository.archive(
            format='tarball',
            path=dest,
            ref=self.version
        )
        if result is False:
            raise exceptions.ArchiveNotFound(
                repo=self.repository.clone_url,
                version=self.version
            )

    def get_owner(self, url):
        """
        Get owner name from repo url

        :param url: url to repository
        :return: repository owner
        """
        # Split owner from url
        owner = url.split("/")[3]
        return owner

    def get_repo_name(self, url):
        """
        Get repo name from repo url

        :param url: url to repository
        :return: repository name
        """
        if url[-4:] == ".git":
            # Remove .git from url
            url = url[:-4]
        # Split repo_name from url
        repo_name = url.split("/")[4]
        return repo_name

    def get_commit_hash(self):
        """
        Get commit hash for this source

        :return: commit hash for source
        """
        commits = self.repository.commits(
            sha=self.version,
            number=1
        )
        # Get first and only (number=1) item from iterator
        commit = next(commits)
        return commit.sha



#  Copyright (c) 2015-2018 Cisco Systems, Inc.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.

import logging
import sys

import colorama

colorama.init(autoreset=True)

SUCCESS = 100
OUT = 101


class LogFilter(object):
    """
    A custom log filter which excludes log messages above the logged
    level.
    """

    def __init__(self, level):
        self.__level = level

    def filter(self, logRecord):  # pragma: no cover
        # https://docs.python.org/3/library/logging.html#logrecord-attributes
        return logRecord.levelno <= self.__level


class CustomLogger(logging.getLoggerClass()):
    """
    A custom logging class which adds additional methods to the logger.  These
    methods serve as syntactic sugar for formatting log messages.
    """

    def __init__(self, name, level=logging.NOTSET):
        super(logging.getLoggerClass(), self).__init__(name, level)
        logging.addLevelName(SUCCESS, 'SUCCESS')
        logging.addLevelName(OUT, 'OUT')

    def success(self, msg, *args, **kwargs):
        if self.isEnabledFor(SUCCESS):
            self._log(SUCCESS, msg, args, **kwargs)

    def out(self, msg, *args, **kwargs):
        if self.isEnabledFor(OUT):
            self._log(OUT, msg, args, **kwargs)


class TrailingNewlineFormatter(logging.Formatter):
    """
    A custom logging formatter which removes additional newlines from messages.
    """

    def format(self, record):
        if record.msg:
            record.msg = record.msg.rstrip()
        return super(TrailingNewlineFormatter, self).format(record)


def get_logger(name=None):
    """
    Build a logger with the given name and returns the logger.

    :param name: The name for the logger. This is usually the module
                 name, ``__name__``.
    :return: logger object
    """
    logging.setLoggerClass(CustomLogger)

    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)

    logger.addHandler(_get_info_handler())
    logger.addHandler(_get_out_handler())
    logger.addHandler(_get_warn_handler())
    logger.addHandler(_get_error_handler())
    logger.addHandler(_get_critical_handler())
    logger.addHandler(_get_success_handler())
    logger.propagate = False

    return logger


def _get_info_handler():
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(logging.INFO)
    handler.addFilter(LogFilter(logging.INFO))
    handler.setFormatter(TrailingNewlineFormatter(cyan_text('%(message)s')))

    return handler


def _get_out_handler():
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(OUT)
    handler.addFilter(LogFilter(OUT))
    handler.setFormatter(TrailingNewlineFormatter('    %(message)s'))

    return handler


def _get_warn_handler():
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(logging.WARN)
    handler.addFilter(LogFilter(logging.WARN))
    handler.setFormatter(TrailingNewlineFormatter(yellow_text('%(message)s')))

    return handler


def _get_error_handler():
    handler = logging.StreamHandler(sys.stderr)
    handler.setLevel(logging.ERROR)
    handler.addFilter(LogFilter(logging.ERROR))
    handler.setFormatter(TrailingNewlineFormatter(red_text('%(message)s')))

    return handler


def _get_critical_handler():
    handler = logging.StreamHandler(sys.stderr)
    handler.setLevel(logging.CRITICAL)
    handler.addFilter(LogFilter(logging.CRITICAL))
    handler.setFormatter(
        TrailingNewlineFormatter(red_text('ERROR: %(message)s')))

    return handler


def _get_success_handler():
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(SUCCESS)
    handler.addFilter(LogFilter(SUCCESS))
    handler.setFormatter(TrailingNewlineFormatter(green_text('%(message)s')))

    return handler


def red_text(msg):
    return color_text(colorama.Fore.RED, msg)


def yellow_text(msg):
    return color_text(colorama.Fore.YELLOW, msg)


def green_text(msg):
    return color_text(colorama.Fore.GREEN, msg)


def cyan_text(msg):
    return color_text(colorama.Fore.CYAN, msg)


def color_text(color, msg):
    return '{}{}{}'.format(color, msg, colorama.Style.RESET_ALL)



class ArchiveNotFound(Exception):
    """
    Exception thrown when archive not found
    """

    def __init__(self, repo, version):
        """
        Inits ArchiveNotFound

        :param repo: url of the repository
        :param version: version of the archive
        """
        self.repo = repo
        self.version = version

    def __str__(self):
        return "Archive not found: {repo}@{version}".format(
            repo=self.repo,
            version=self.version)


class RepoNotFound(Exception):
    """
    Exception thrown when repository not found
    """

    def __init__(self, repo):
        """
        Inits RepoNotFound

        :param repo: url of the repository
        """
        self.repo = repo

    def __str__(self):
        """
        Get error message

        :return: Error message as string
        """
        return "Repository not found: {repo}".format(repo=self.repo)


class ValidationError(Exception):
    """
    Exception thrown when authentication error with source
    """

    def __init__(self, errors):
        """
        Inits ValidationError

        :param errors: errors as dict
        """
        self.errors = errors

    def __str__(self):
        """
        Get error message

        :return: Error message as string
        """
        return "Validation errors: {errors}".format(errors=self.errors)


class AuthenticationError(Exception):
    """
    Exception thrown when authentication error with source
    """

    def __init__(self, url):
        """
        Inits AuthenticationError

        :param url: url of the server
        """
        self.url = url

    def __str__(self):
        """
        Get error message

        :return: Error message as string
        """
        return "Authentication error with: {url}".format(url=self.url)


class HttpError(Exception):
    """
    Exception thrown when the return code is not 2xx
    """

    def __init__(self, url, code=None):
        """
        Inits HttpError

        :param url: url of the request
        """
        self.url = url
        self.code = code

    def __str__(self):
        """
        Get error message

        :return: Error message as string
        """
        return "Request could not complete for: {url}".format(url=self.url)


class DirectoryDoesNotExist(Exception):
    """
    Exception thrown when directory does not exist
    """

    def __init__(self, directory):
        """
        Inits DirectoryDoesNotExist

        :param directory: path of non existing directory
        """
        self.directory = directory

    def __str__(self):
        """
        Get error message

        :return: Error message as string
        """
        return "Directory does not exist: {directory}".format(
            directory=self.directory
        )


class DirectoryNotEmpty(Exception):
    """
    Exception thrown when directory not empty
    """

    def __init__(self, directory):
        """
        Inits DirectoryNotEmpty

        :param directory: path of not empty directory
        """
        self.directory = directory

    def __str__(self):
        """
        Get error message

        :return: Error message as string
        """
        return "Directory not empty: {directory}".format(
            directory=self.directory
        )


class ParserError(Exception):
    """
    Exception thrown when parsing does not complete
    """

    def __init__(self, file, error):
        """
        Inits ParserError

        :param file: path to file which has errors
        :param error: error message of parser
        """
        self.file = file
        self.error = error

    def __str__(self):
        """
        Get error message

        :return: Error message as string
        """
        return "Parsing failed for {file} due to {error}".format(
            file=self.file,
            error=self.error)



import click

from gip import command


@click.group()
@click.option(
    '--debug/--no-debug',
    default=False,
    help='Enable or disable debug mode. Default is disabled.'
)
@click.option(
    '--gitlab-token',
    envvar='GIP_GITLAB_TOKEN',
    default=False,
    help='Provide the private token for the Gitlab API. \
        Can be set as environment variable GIP_GITLAB_TOKEN'
)
@click.option(
    '--github-token',
    envvar='GIP_GITHUB_TOKEN',
    default=False,
    help='Provide the private token for the Github API. \
        Can be set as environment variable GIP_GITHUB_TOKEN'
)
@click.option(
    '--lock-file',
    default='.giplock.yml',
    help='Provide the path to the lockfile, defaults to \
        .giplock.yml in the current working directory',
    type=click.Path(
        writable=True
    )
)
@click.version_option(version='0.0.1')
@click.pass_context
def main(ctx, debug, gitlab_token, github_token, lock_file):
    """
    \b
     _______ __
    |     __|__|.-----.
    |    |  |  ||  _  |
    |_______|__||   __|
                |__|
    Gip is a language agnostic dependency manager
    which uses API calls to pull repositories.

    Enable autocomplete for Bash (.bashrc):
      eval "$(_GIP_COMPLETE=source gip)"

    Enable autocomplete for ZSH (.zshrc):
      eval "$(_GIP_COMPLETE=source_zsh gip)"
    """
    ctx.obj = {}
    ctx.obj['args'] = {}
    ctx.obj['args']['debug'] = debug
    ctx.obj['args']['gitlab_token'] = gitlab_token
    ctx.obj['args']['github_token'] = github_token
    ctx.obj['args']['lock_file'] = lock_file


main.add_command(command.install.install)



import yaml
import sys
import pathlib

from gip import logger
from gip import exceptions

LOG = logger.get_logger(__name__)


def read_yaml(path):
    """
    Parse YAML file to Python object

    :param path: path to yamlfile
    :return: python object
    """
    # TODO: Rewrite to with open
    try:
        file_stream = open(path, 'r')
    except OSError as e:
        LOG.exception(e)
    else:
        try:
            return yaml.safe_load(file_stream)
        except yaml.YAMLError as e:
            raise exceptions.ParserError(
                file=path,
                error=e)
        finally:
            file_stream.close()


def write_yaml(path, data):
    """
    Write Python object to YAML

    :param path: path to yamlfile
    """
    # TODO: Rewrite to with open
    try:
        file_stream = open(path, 'w')
    except OSError as e:
        LOG.exception(e)
    else:
        try:
            yaml.safe_dump(
                data,
                stream=file_stream,
                default_flow_style=False
            )
        except yaml.YAMLError as e:
            raise exceptions.ParserError(
                file=path,
                error=e)
        finally:
            file_stream.close()


def sysexit(code=1):
    sys.exit(code)


def sysexit_with_message(msg, code=1):
    LOG.critical(msg)
    sysexit(code)


def remove_file(path):
    """
    Remove file from disk

    :param path: path to file to remove
    """
    # TODO: fix some exception handling
    pathlib.Path(path).unlink()


def merge_dicts(*dict_args):
    """
    Given any number of dicts, shallow copy and merge into a new dict,
    precedence goes to key value pairs in latter dicts.
    """
    result = {}
    for dictionary in dict_args:
        result.update(dictionary)
    return result



import shutil
import click
import pathlib

from gip import logger
from gip import util
from gip import exceptions
from gip import model
from gip import sources

LOG = logger.get_logger(__name__)


@click.command()
@click.pass_context
@click.option(
    '--upgrade',
    help='Bring versions to what is specified in requirements.yml',
    type=bool
)
@click.option(
    '--requirements',
    '-r',
    help='Requirements file to install',
    type=click.Path(
        exists=True,
    )
)
def install(ctx, upgrade, requirements):
    """
    Install dependencies, if already present just skips when version checking is required, pass --upgrade.  # noqa:E501

    :param ctx: click object containing the arguments from global
    :param upgrade: boolean whenever upgrading is enabled
    :param requirements: click path object poiting to requirements file
    """
    args = ctx.obj.get('args')

    # Parse requirements file to Python object
    requirements = _parse_and_validate(
        path=requirements,
        type='requirements'
    )

    # After parsing check if a token is mandatory or advised
    if (any(d['type'] == 'gitlab' for d in requirements) and
            args['gitlab_token'] is None):
        util.sysexit_with_message(
            "Gitlab repo in requirements but no token passed, \
            use --gitlab-token"
        )
    if (any(d['type'] == 'github' for d in requirements) and
            args['github_token'] is None):
        util.sysexit_with_message(
            "Github repo in requirements but no token passed, \
            use --github-token"
        )

    # Init lockfile
    locks_path = pathlib.Path(args['lock_file']).resolve()

    # Init current_lock list
    current_lock = {}
    if locks_path.is_file():
        current_lock = _parse_and_validate(
            path=locks_path,
            type='locks'
        )
    else:
        LOG.info("Lock file does not exist, will be created at {path}".format(
            path=locks_path)
        )

    # Init new_lock list
    new_lock = {}
    # Loop requirements for downloading
    for requirement in requirements:
        if requirement['name'] in current_lock and not upgrade:
            LOG.info("{requirement} already installed, skipping".format(
                requirement=requirement['name'])
            )
        else:
            try:
                if requirement['type'] == 'gitlab':
                    # Init Gitlab object
                    source = sources.gitlab.Gitlab(
                        repo=requirement['repo'],
                        version=requirement.get('version'),
                        token=args['gitlab_token']
                    )
                elif requirement['type'] == 'github':
                    # Init Github object
                    source = sources.github.Github(
                        repo=requirement['repo'],
                        version=requirement.get('version'),
                        token=args['github_token']
                    )
            except (exceptions.RepoNotFound,
                    exceptions.HttpError,
                    exceptions.AuthenticationError) as e:
                util.sysexit_with_message(e)

            if source.get_commit_hash() == \
                    current_lock.get(requirement['name']):
                LOG.info("{requirement} already the current version, \
                    skipping".format(
                        requirement=requirement['name'])
                    )
            else:
                # Convert dest to absolute path
                dest = pathlib.Path(requirement.get('dest', '')).resolve()
                # Create archive name (ex. ansible-role-plex.tar.gz)
                archive_name = "{}.tar.gz".format(requirement['name'])
                # Append name to destination directory
                archive_dest = dest.joinpath(archive_name)

                # Remove old versions if existing
                test = dest.joinpath(requirement['name'])
                if test.is_dir():
                    shutil.rmtree(test)

                # Get the archive
                try:
                    source.get_archive(
                        dest=archive_dest
                    )
                except (exceptions.DirectoryDoesNotExist,
                        exceptions.AuthenticationError,
                        exceptions.ArchiveNotFound) as e:
                    # Write current state to lock
                    _write_lock_file(
                        path=locks_path,
                        current_lock=current_lock,
                        new_lock=new_lock
                    )
                    util.sysexit_with_message(e)

                # Extract archive to location
                try:
                    source.extract_archive(
                        src=archive_dest,
                        dest=dest,
                        name=requirement['name']
                    )
                except (exceptions.DirectoryDoesNotExist,
                        TypeError,
                        FileNotFoundError) as e:
                    # Write current state to lock
                    _write_lock_file(
                        path=locks_path,
                        current_lock=current_lock,
                        new_lock=new_lock
                    )
                    # Cleanup archive
                    util.remove_file(archive_dest)
                    # Exit with message
                    util.sysexit_with_message(e)

                # No exceptions add to new_lock since succesfull download
                if requirement['name'] in current_lock:
                    LOG.success("{requirement} successfully updated to \
                        {version}".format(
                            requirement=requirement['name'],
                            version=source.version
                        )
                    )
                else:
                    LOG.success("{requirement} successfully installed".format(
                        requirement=requirement['name'])
                    )
                new_lock[requirement['name']] = source.get_commit_hash()

    # End for loop
    _write_lock_file(
        path=locks_path,
        current_lock=current_lock,
        new_lock=new_lock
    )


def _write_lock_file(path, current_lock, new_lock):
    """
    Write current state to lock file

    :param path: path of the current lock file and where to write to
    :param current_lock: dict containing the lock before executing gip
    :param new_lock: dict container the new lock
    """
    # Check if new_lock has data
    if new_lock:
        # No current lock just write to file
        if not current_lock:
            util.write_yaml(
                path=path,
                data=new_lock
            )
        else:
            # Current lock, merge the two and write to file
            util.write_yaml(
                path=path,
                data=util.merge_dicts(current_lock, new_lock)
            )


def _parse_and_validate(path, type):
    """
    Parse and validate file against Cerberus scheme

    :param path: path to file
    :param type: one of the modeltypes available in model.scheme
    :return: parsed and validated data
    """
    try:
        data = util.read_yaml(path=path)
    except exceptions.ParserError as e:
        util.sysexit_with_message(e)

    # Validate requirements file
    try:
        model.scheme.validate(
            type=type,
            data=data
        )
    except exceptions.ValidationError as e:
        util.sysexit_with_message(
            "Parsing failed for {file} due to {errors}".format(
                file=path,
                errors=e.errors
            )
        )

    # Return parsed and validated data
    return data



import cerberus
from gip import exceptions

models = {
    'locks': {
        'dict': {
            'type': 'dict',
            'keyschema': {
                'type': 'string',
                'regex': '^[a-z-]+$'
            },
            'valueschema': {
                'type': 'string',
                'regex': '^[0-9a-z]+$'
            },
            'allow_unknown': True
        }
    },
    'requirements': {
        'dict': {
            'type': 'list',
            'schema': {
                'type': 'dict',
                'schema': {
                    'name': {
                        'type': 'string',
                        'regex': '^[a-z-]+$',
                        'required': True
                    },
                    'repo': {
                        'type': 'string',
                        'required': True
                    },
                    'type': {
                        'type': 'string',
                        'required': True,
                        'allowed': [
                            'gitlab',
                            'github'
                        ]
                    },
                    'version': {
                        'type': 'string'
                    },
                    'dest': {
                        'type': 'string'
                    }
                }
            }
        }
    }
}


def validate(type, data):
    """
    Validate types against scheme

    :param type: one of the keys in the models dict
    :param data: data to validate against the scheme
    :raise exceptions.ValidationError: if data not valid
    """
    try:
        validator = cerberus.Validator(models[type])
    except KeyError:
        raise exceptions.ValidationError(errors={
            'No matching type found for validation'
        })

    # Add data to dict, Cerberus only works with dicts, not lists
    if validator.validate({'dict': data}):
        return True  # Valid data according to scheme
    else:
        raise exceptions.ValidationError(errors=validator.errors)



import pathlib
import tarfile

from gip import logger
from gip import exceptions

LOG = logger.get_logger(__name__)


class Source():
    """
    Superclass which interfaces all the sources
    """

    def get_commit_hash(self):
        raise NotImplementedError

    def get_archive(self, dest):
        raise NotImplementedError

    def extract_archive(self, src, dest, name, remove_src=True):
        """
        Extract archive to destination, zip and tar supported

        :param src: path to archive to extract
        :param dest: directory where to extract the archive to
        :param name: name of the folder of the extracted archive
        :param remove_src: boolean to enable removal of the source file
        :raise exceptions.DirectoryDoesNotExist: directory does not exist
        :raise FileNotFoundError: source does not exist
        :raise TypeError: archive is of an unsupported type
        """
        src = pathlib.Path(src)
        dest = pathlib.Path(dest)

        # Raise error when destination directory does not exists
        if not dest.is_dir():
            raise exceptions.DirectoryDoesNotExist(dest)

        # Raise error when archive does not exists
        if not src.is_file():
            raise FileNotFoundError("Archive not found: {archive}".format(
                            archive=src
                        ))

        if tarfile.is_tarfile(src):
            archive = tarfile.open(src)
            archive.extractall(path=dest)
            if remove_src:
                # No need for try/except only raises on directory.
                src.unlink()

            # Rename extracted folder when name is passed
            if name:
                extracted_folder_name = archive.getmembers()[0].name
                dest = pathlib.PurePath(dest)
                extracted_archive = pathlib.Path(
                                        dest.joinpath(extracted_folder_name))

                # Thrown error when for some reaseon the extraction has failed
                if not extracted_archive.is_dir():
                    raise exceptions.DirectoryDoesNotExist(extracted_archive)

                extracted_archive.rename(dest.joinpath(name))
        else:
            raise TypeError("Downloaded archive is of a non supported \
                            archive type: {archive}".format(
                                archive=src
                            ))
